SystemOrganization addCategory: #EmergencyEvaluator!

ReadWriteStream subclass: #Transcripter
	instanceVariableNames: 'frame para'
	classVariableNames: ''
	poolDictionaries: 'EventSensorConstants'
	category: 'EmergencyEvaluator'!

!Transcripter commentStamp: 'VeronicaUquillas 6/11/2010 13:45' prior: 0!
Transcripter is a dog-simple scrolling stream with display.  It is intended to operate with no support from MVC or color in a minimal, or headless version of Pharo.  No attention has been paid to appearance or performance.!

!Transcripter class methodsFor: 'utilities' stamp: 'AlainPlantec 9/12/2011 01:23'!
askForEmergencyEvaluatorOrExitWithText: aText 
	| m |
	m := String
				streamContents: [:s | 
					s nextPutAll: aText.
					s cr; nextPutAll: '-----------------------------------------------------------------'.
					s cr; nextPutAll: 'Type CR to enter an emergency evaluator.'.
					s cr; nextPutAll: 'Type any other character to restart.'].
	Display getCanvas 
		drawString: m
		from: 1
		to: m size
		autoBoundAt: 0 @ 0
		font: nil
		color: Color black.
	self nextChar = Character cr
		ifTrue: [self emergencyEvaluator]! !

!Transcripter class methodsFor: 'utilities' stamp: 'AlainPlantec 9/11/2011 23:10'!
emergencyEvaluator
	(self newInFrame: (0@0 corner: 320@200))
		show: 'Type ''revert'' to revert your last method change.
Type ''exit'' to exit the emergency evaluator.';
		readEvalPrint! !

!Transcripter class methodsFor: 'instance creation' stamp: 'AlainPlantec 9/11/2011 23:27'!
newInFrame: frame 
	"
	(Transcripter newInFrame: (0@0 extent: 100@200))
	nextPutAll: 'Hello there'; endEntry;
	cr; print: 355.0/113; endEntry;
	readEvalPrint
	"
	| transcript |
	transcript := self 
				on: (String new: 100).
	transcript initInFrame: frame.
	^ transcript clear! !

!Transcripter class methodsFor: 'utilities' stamp: 'AlainPlantec 9/12/2011 01:41'!
nextChar
	[| evt type | 
		(Delay forMilliseconds: 20) wait.
		Sensor nextEvent
		ifNotNil: [:evtBuf | 
			type := evtBuf first.
			type = EventTypeKeyboard
				ifTrue: [evt := World currentHand generateKeyboardEvent: evtBuf.
					evt isKeyDown
						ifTrue: [| c |
							c := evt keyCharacter.
							(c asciiValue < 128
									or: [c isSeparator
											or: [c isSpecial
													or: [c = Character backspace]]])
								ifTrue: [Sensor flushKeyboard.
									^ evt keyCharacter]]]]] repeat! !

!Transcripter class methodsFor: 'instance creation' stamp: 'AlainPlantec 9/11/2011 23:15'!
startTranscriptProcess
	"Transcripter startTranscriptProcess"
	| activeProcess |
	Transcript := self newInFrame: Display boundingBox.
	activeProcess := [Transcript readEvalPrint.
			Smalltalk processShutDownList: true;
				 quitPrimitive] newProcess priority: Processor userSchedulingPriority.
	activeProcess resume.
	Processor terminateActive ! !

!Transcripter methodsFor: 'private' stamp: 'di 8/14/97 12:12'!
black
	Display depth = 1 ifTrue: [^ Bitmap with: 16rFFFFFFFF "Works without color support"].
	^ Color black! !

!Transcripter methodsFor: 'accessing' stamp: 'AlainPlantec 9/12/2011 01:43'!
clear
	self clearFrame.
	self
		on: (String new: 100);
		 endEntry ! !

!Transcripter methodsFor: 'private' stamp: 'AlainPlantec 9/12/2011 01:43'!
clearFrame
	Display getCanvas
		frameAndFillRectangle: frame
		fillColor: self white
		borderWidth: 2
		borderColor: self black! !

!Transcripter methodsFor: 'accessing' stamp: 'AlainPlantec 9/12/2011 00:30'!
compose 
	para
		compose: self contents asText 
		style: self textStyle
		from: 1
		in: ((frame insetBy: 4) withHeight: 9999).
! !

!Transcripter methodsFor: 'command line' stamp: 'AlainPlantec 9/12/2011 01:37'!
confirm: queryString 
	| choice |
	[true]
		whileTrue: 
			[choice := self request: queryString , '
Please type yes or no followed by return
> '.
			choice first asUppercase = $Y ifTrue: [^ true].
			choice first asUppercase = $N ifTrue: [^ false]]! !

!Transcripter methodsFor: 'accessing' stamp: 'AlainPlantec 9/12/2011 01:43'!
display
	self clearFrame.
	Display getCanvas
		paragraph: para
		bounds: frame
		color: self black! !

!Transcripter methodsFor: 'accessing' stamp: 'AlainPlantec 9/12/2011 00:35'!
endEntry
	| c d cb |
	c := self contents.
	Display extent ~= DisplayScreen actualScreenSize
		ifTrue: ["Handle case of user resizing physical window"
			DisplayScreen startUp.
			frame := frame intersect: Display boundingBox.
			^ self clear; show: c].
	self compose.
	d := para extent y - frame bottom.
	d > 0
		ifTrue: ["Scroll up to keep all contents visible"
			cb := para characterBlockAtPoint: para compositionRectangle topLeft + (0 @ (d + self textStyle lineGrid)).
			self on: (c copyFrom: cb stringIndex to: c size).
			readLimit := position := collection size.
			^ self endEntry].
	self display! !

!Transcripter methodsFor: 'initialization' stamp: 'AlainPlantec 9/15/2011 17:27'!
initInFrame: rect
	frame := rect.
	para := Paragraph new.
	self compose! !

!Transcripter methodsFor: 'command line' stamp: 'AlainPlantec 9/11/2011 23:58'!
nextChar
	^ self class nextChar! !

!Transcripter methodsFor: 'command line' stamp: 'AlainPlantec 9/12/2011 01:38'!
readEvalPrint
	| line okToRevert |
	okToRevert := true.
	Sensor flushKeyboard.
	[#('quit' 'exit' 'done' ) includes: (line := self request: '> ')]
		whileFalse: [line = 'revert'
				ifTrue: [okToRevert
						ifTrue: [(self confirm: 'Revert: ' , RecentMessageList uniqueInstance lastEntry asString , ' ?')
								ifTrue: [RecentMessageList uniqueInstance revertLastMethodSubmission.
									self cr; show: 'reverted: ' , RecentMessageList uniqueInstance lastEntry asString.
									okToRevert := false]]
						ifFalse: [self cr; show: 'Only one level of revert currently supported']]
				ifFalse: [self cr;
						show: ([self class evaluatorClass evaluate: line]
								ifError: [:err | err])]]. 
	self cr; show: 'Done'! !

!Transcripter methodsFor: 'command line' stamp: 'AlainPlantec 9/11/2011 22:36'!
request: prompt 
	| startPos char contents |
	self cr; show: prompt.
	startPos := position.
	Sensor flushKeyboard.
	[(char := self nextChar) = Character cr]
		whileFalse: [char = Character backspace
				ifTrue: [readLimit := position := position - 1 max: startPos]
				ifFalse: [self nextPut: char].
			self endEntry].
	contents := self contents.
	^ contents copyFrom: startPos + 1 to: contents size ! !

!Transcripter methodsFor: 'accessing' stamp: 'sma 2/26/2000 19:35'!
show: anObject
	self nextPutAll: anObject asString; endEntry! !

!Transcripter methodsFor: 'private' stamp: 'AlainPlantec 9/11/2011 16:12'!
textStyle
	^ TextStyle default! !

!Transcripter methodsFor: 'private' stamp: 'di 8/14/97 12:12'!
white
	Display depth = 1 ifTrue: [^ Bitmap with: 0 "Works without color support"].
	^ Color white! !
